import { DeepClient, DeepClientResult, Table } from "../client";
import { MutationInputLink } from "../client_types";
import { IGenerateMutationBuilder, ISerialOptions } from "../gql";
/**
 * A class for building serial transitions
 *
 * @example
```ts
const builder = new SerialTransitionsBuilder({ deep });
const table = 'links';
const transition = [null, {
  type_id: deep.id("@deep-foundation/core", "Type")
}];
const {data} =
  builder
  .append({ table, transition, alias: firstLinkAlias })
  .append({ table, transition, alias: secondLinkAlias })
  .execute()
const firstLink = data[firstLinkAlias] // or data[0]
```
 */
export declare class SerialTransitionsBuilder {
    private deep;
    private serialActions;
    private defaultTable;
    private executeOptions;
    constructor(options: SerialTransitionsBuilderOptions);
    append(options: AppendTransitionOptions): this;
    appendMultiple(options: AppendTransitionOptions[]): this;
    clear(): this;
    execute(options?: ExecuteOptions): Promise<DeepClientResult<Record<string, {
        id: number;
    }>>>;
    actions(): SerialAction[];
    getTransitionType(transition: Transition): TransitionType;
    setDefaultTable(table: Table<'insert' | 'update' | 'delete'>): this;
    getDefaultTable(): Table<"insert" | "update" | "delete">;
    setDeep(deep: DeepClient): this;
    getDeep(): DeepClient<import("../minilinks").Link<number>>;
    setExecuteOptions(options: ExecuteOptions): this;
    getExecuteOptions(): ExecuteOptions;
}
export interface AppendTransitionOptions {
    /**
     * A transition to append
     *
     * If you want to insert link - the first element must be null and the second must be link. If you want to update link - the first and second elements must be links. If you want to delete link - the first element must be link and second must be null
     */
    transition: Transition;
    /**
     * A table name where operation must be executed
     *
     * @defaultValue 'links'
     */
    table?: Table<Transition[0] extends null ? 'insert' : Transition[1] extends null ? 'delete' : 'update'>;
    alias?: string;
}
export type TransitionType = 'insert' | 'update' | 'delete';
export type TransitionItem = MutationInputLink | null;
export type Transition = Array<TransitionItem>;
export type ExecuteOptions = Omit<ISerialOptions, 'actions'>;
export type SerialTransitionsBuilderOptions = {
    deep: DeepClient;
    defaultTable?: Table<'insert' | 'update' | 'delete'>;
    executeOptions?: ExecuteOptions;
};
type SerialAction = {
    mutation: IGenerateMutationBuilder;
    alias?: string;
    index: number;
    transitionType: TransitionType;
    table: Table<'insert' | 'update' | 'delete'>;
};
export {};
