"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SerialTransitionsBuilder = void 0;
const gql_1 = require("../gql");
/**
 * A class for building serial transitions
 *
 * @example
```ts
const builder = new SerialTransitionsBuilder({ deep });
const table = 'links';
const transition = [null, {
  type_id: deep.id("@deep-foundation/core", "Type")
}];
const {data} =
  builder
  .append({ table, transition, alias: firstLinkAlias })
  .append({ table, transition, alias: secondLinkAlias })
  .execute()
const firstLink = data[firstLinkAlias] // or data[0]
```
 */
class SerialTransitionsBuilder {
    deep;
    serialActions;
    defaultTable;
    executeOptions;
    constructor(options) {
        this.deep = options.deep;
        this.serialActions = [];
        this.defaultTable = options.defaultTable ?? 'links';
        this.executeOptions = options.executeOptions ?? {};
    }
    append(options) {
        this.appendMultiple([options]);
        return this;
    }
    appendMultiple(options) {
        for (const optionsItem of options) {
            const { table = this.defaultTable, transition } = optionsItem;
            const transitionType = this.getTransitionType(transition);
            const index = this.serialActions.length;
            let serialAction;
            switch (transitionType) {
                case 'insert':
                    serialAction = {
                        mutation: (0, gql_1.insertMutation)(table, { objects: transition[1] }),
                        index,
                        transitionType,
                        table
                    };
                    break;
                case 'update':
                    serialAction = {
                        mutation: (0, gql_1.updateMutation)(table, { exp: transition[0], value: transition[1] }),
                        index,
                        transitionType,
                        table
                    };
                    break;
                case 'delete':
                    serialAction = {
                        mutation: (0, gql_1.deleteMutation)(table, { exp: transition[0] }),
                        index,
                        transitionType,
                        table
                    };
                default:
                    throw new Error('Invalid transition type. If you want to insert link - the first element must be null and the second must be link. If you want to update link - the first and second elements must be links. If you want to delete link - the first element must be link and second must be null');
            }
            serialAction.alias = optionsItem.alias ?? `${transitionType}_${table}_${index}`;
            this.serialActions.push(serialAction);
        }
        return this;
    }
    clear() {
        this.serialActions = [];
        return this;
    }
    async execute(options = this.executeOptions) {
        const result = await this.deep.apolloClient.mutate((0, gql_1.generateSerial)({
            actions: this.serialActions.map(serialAction => serialAction.mutation),
            ...options
        }));
        const data = result.data;
        for (const serialAction of this.serialActions) {
            const oldKey = `m${serialAction.index}`;
            const newValue = {
                ...data[oldKey].returning,
                index: serialAction.index
            };
            data[serialAction.alias] = newValue;
            data[serialAction.index] = newValue;
            delete data[oldKey];
        }
        // @ts-ignore
        return {
            ...result,
            data,
        };
    }
    actions() {
        return this.serialActions;
    }
    getTransitionType(transition) {
        if (transition[0] === null) {
            return 'insert';
        }
        else if (transition[1] === null) {
            return 'delete';
        }
        else if (transition[0] !== null && transition[1] !== null) {
            return 'update';
        }
        else {
            throw new Error('Invalid transition');
        }
    }
    setDefaultTable(table) {
        this.defaultTable = table;
        return this;
    }
    getDefaultTable() {
        return this.defaultTable;
    }
    setDeep(deep) {
        this.deep = deep;
        return this;
    }
    getDeep() {
        return this.deep;
    }
    setExecuteOptions(options) {
        this.executeOptions = options;
        return this;
    }
    getExecuteOptions() {
        return this.executeOptions;
    }
}
exports.SerialTransitionsBuilder = SerialTransitionsBuilder;
//# sourceMappingURL=serial-transitions-builder.js.map