import { DeepClientInstance } from './client';
/**
 * Represents a deep package
 *
 * @remarks
 * This class intended to be extended by packages
 *
 * @example
```ts
const package = new Package({deep, name});
const {name: packageName} = package;
const batteryLevelValueLinkId = await package.batteryLevelValue.id();
```
  */
export declare class Package {
    deep: DeepClientInstance;
    /**
     * Name of the package
     */
    name: string;
    constructor(options: PackageOptions);
    /**
     * Creates an entity
     *
     * @example
     * #### Create an entity
  ```ts
  class MyPackage extends Package {
    public yourLinkName = this.createEntity("YourLinkName");
  }
  const myPackage = new MyPackage({deep});
  const myLinkId = await myPackage.yourLinkName.id();
  const myLinkLocalId = await myPackage.yourLinkName.idLocal();
  ```
     */
    createEntity(name: string): {
        /**
         * Gets id of the link
         *
         * @example
         * #### Get id of the link
  ```ts
  const myPackage = new MyPackage({deep});
  const myLinkId = await myPackage.yourLinkName.id();
  ```
         */
        id: () => Promise<number>;
        /**
         * Gets id of the link from minilinks
         *
         * @example
         * #### Get id of the link from minilinks
  ```ts
  const myPackage = new MyPackage({deep});
  const myLinkLocalId = await myPackage.yourLinkName.idLocal();
  ```
         */
        idLocal: () => number;
        /**
         * Name of the link
         */
        name: string;
    };
    /**
     * Gets id of the package link
     *
     * @example
     * #### Get id of the package link
  ```ts
  const package = new Package({deep});
  const myLinkId = await package.id("MyLinkName");
  ```
     */
    id(...names: string[]): Promise<number>;
    /**
   * Gets id of the package link from minilinks
   *
   * @example
   * #### Get id of the package link from minilinks
```ts
const package = new Package({deep});
await package.applyMiniLinks();
const myLinkId = await package.idLocal("MyLinkName");
```
   */
    idLocal(...names: string[]): number;
    /**
     * Pastes your links into minilinks
     *
     * @example
     * #### Use applyMiniLinks and idLocal
  ```ts
  const package = new Package({deep});
  await package.applyMiniLinks();
  const deviceLinkId = await package.Device.idLocal();
  ```
     */
    applyMiniLinks(): Promise<{
        errors?: import("./minilinks").MinilinkError[];
        anomalies?: import("./minilinks").MinilinkError[];
    }>;
}
export interface PackageOptions {
    name: string;
    deep: DeepClientInstance;
}
