"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Package = void 0;
const debug_js_1 = require("./debug.js");
const moduleLog = debug_js_1.debug.extend('package');
/**
 * Represents a deep package
 *
 * @remarks
 * This class intended to be extended by packages
 *
 * @example
```ts
const package = new Package({deep, name});
const {name: packageName} = package;
const batteryLevelValueLinkId = await package.batteryLevelValue.id();
```
  */
class Package {
    deep;
    /**
     * Name of the package
     */
    name;
    constructor(options) {
        const log = moduleLog.extend(this.name);
        log({ options });
        this.deep = options.deep;
        this.name = options.name;
    }
    /**
     * Creates an entity
     *
     * @example
     * #### Create an entity
  ```ts
  class MyPackage extends Package {
    public yourLinkName = this.createEntity("YourLinkName");
  }
  const myPackage = new MyPackage({deep});
  const myLinkId = await myPackage.yourLinkName.id();
  const myLinkLocalId = await myPackage.yourLinkName.idLocal();
  ```
     */
    createEntity(name) {
        const log = moduleLog.extend(this.createEntity.name);
        log({ name });
        const result = {
            /**
             * Gets id of the link
             *
             * @example
             * #### Get id of the link
      ```ts
      const myPackage = new MyPackage({deep});
      const myLinkId = await myPackage.yourLinkName.id();
      ```
             */
            id: async () => {
                const log = moduleLog.extend(this.id.name);
                const result = await this.id(name);
                log({ result });
                return result;
            },
            /**
             * Gets id of the link from minilinks
             *
             * @example
             * #### Get id of the link from minilinks
      ```ts
      const myPackage = new MyPackage({deep});
      const myLinkLocalId = await myPackage.yourLinkName.idLocal();
      ```
             */
            idLocal: () => {
                return this.idLocal(name);
            },
            /**
             * Name of the link
             */
            name: name
        };
        log({ result });
        return result;
    }
    /**
     * Gets id of the package link
     *
     * @example
     * #### Get id of the package link
  ```ts
  const package = new Package({deep});
  const myLinkId = await package.id("MyLinkName");
  ```
     */
    async id(...names) {
        const log = moduleLog.extend(this.id.name);
        log({ names });
        const deepIdArgs = [this.name, ...names];
        log({ deepIdArgs });
        const result = await this.deep.id(...deepIdArgs);
        log({ result });
        return result;
    }
    /**
   * Gets id of the package link from minilinks
   *
   * @example
   * #### Get id of the package link from minilinks
```ts
const package = new Package({deep});
await package.applyMiniLinks();
const myLinkId = await package.idLocal("MyLinkName");
```
   */
    idLocal(...names) {
        const log = moduleLog.extend(this.idLocal.name);
        log({ names });
        const deepIdLocalArgs = [this.name, ...names];
        log({ deepIdLocalArgs });
        const result = this.deep.idLocal(...deepIdLocalArgs);
        log({ result });
        return result;
    }
    /**
     * Pastes your links into minilinks
     *
     * @example
     * #### Use applyMiniLinks and idLocal
  ```ts
  const package = new Package({deep});
  await package.applyMiniLinks();
  const deviceLinkId = await package.Device.idLocal();
  ```
     */
    async applyMiniLinks() {
        const log = moduleLog.extend(this.applyMiniLinks.name);
        const { data: packageLinks } = await this.deep.select({
            up: {
                tree_id: {
                    _id: ["@deep-foundation/core", 'containTree']
                },
                parent_id: {
                    _id: [this.name]
                }
            }
        });
        log({ packageLinks });
        if (!packageLinks) {
            throw new Error(`Package with name ${this.name} is not found`);
        }
        const result = this.deep.minilinks.apply(packageLinks);
        log({ result });
        return result;
    }
}
exports.Package = Package;
//# sourceMappingURL=package.js.map